########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, NM, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################
#
# toplevel window name creator
#
set topleveln 0
proc toplevel-window-name {} {
    global topleveln
    return .w[incr topleveln]
}
#
# textview - put a text widget and associated scrollbars
# into an existing window.  The text widget will be $w.t
#
proc textview {w args} {
    grid [eval text $w.t $args] -row 0 -col 0 -sticky nsew
    $w.t configure -xscrollcommand [list $w.x set]
    $w.t configure -yscrollcommand [list $w.y set]
    grid [scrollbar $w.y -orient vertical] -row 0 -col 1 -sticky ns
    $w.y configure -command [list $w.t yview]
    grid [scrollbar $w.x -orient horizontal] -row 1 -col 0 -sticky ew
    $w.x configure -command [list $w.t xview]
    grid columnconfigure $w 0 -weight 1
    grid columnconfigure $w 1 -weight 0
    grid rowconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 0
    return $w
}
#
# readfile - open a file with specified encoding,
# read it into a string, close the file, and
# return the string
#
proc readfile {file {encoding {}}} {
    set fp [open $file r]
    if {[string length $encoding]} {
	fconfigure $fp -encoding $encoding
    }
    set contents [read $fp]
    close $fp
    return $contents
}
#    
# browse the contents of a file
# usually a file in edict or kanjidic format
#
proc browsefile {title file {font {}} {margin {}} {encoding euc-jp} {margin 0}} {
    browsedata $title [readfile $file $encoding] $font $margin
}
#    
# browse some immediate data
#
proc browsedata {title data {font {}} {margin 0}} {
    set w [toplevel-window-name]
    toplevel $w
    wm title $w $title
    textview $w -wrap none
    if {[string length $font] > 0} {
	$w.t configure -font $font
    }
    $w.t insert end $data
    if {$margin} {
    }
}
#
# shuffle a list
#
proc shuffle {list} {
    set n 0
    foreach x $list {
	set t($n) $x
	incr n
    }
    set list {}
    for {set i 0} {$i < $n} {incr i} {
	set j [expr $i+[random]%($n-$i)]
	lappend list $t($j)
	set t($j) $t($i)
    }
    return $list
}
#
# pick a card, any card
#
proc choose-one {list} {
    return [lindex $list [expr {[random]%[llength $list]}]]
}
