########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, NM, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################
#
# display record in raw form
#
proc tkj-browse-record {k title} {
    set w [tkj-toplevel $title]

    grid [frame $w.x -border 5 -relief ridge] -row 0 -column 0 -rowspan 2 -sticky ns
    grid [label $w.k -font [tkj-font 48] -text $k -border 5 -relief ridge] -row 0 -column 1 -sticky nsew
    grid [frame $w.d] -row 0 -column 2 -rowspan 2 -sticky ns
    grid [frame $w.r -border 5 -relief ridge] -row 1 -column 1 -sticky ns

    foreach {name value} [get-record $k] {
	lappend keep($name) $value
    }
    # 
    foreach {name row} {
	B 0
	C 1
	S 2
	F 3
	G 4
	P 5
	I 6
	Q 7
    } {
	if {[info exists keep($name)]} {
	    set win $w.r.[string tolower $name]
	    grid [label $win -text "[tkj-get-field-label $name]: [join $keep($name) ,]"] -row $row -sticky w
	    if {[string compare $name G] == 0} {
		$win configure -text "[tkj-get-field-value-label $name $keep($name)]"
	    }
	}
    }
    foreach {name row} {
	MN 0
	MP 1
	U 2
	J0 3
	J1 3
	N 4
	V 5
	H 6
	DR 7
	E 8
	IN 9
	K 10
	L 11
	O 12
    } {
	if {[info exists keep($name)]} {
	    set win $w.x.[string tolower $name]
	    grid [label $win -text "[tkj-get-field-label $name]: [join $keep($name) ,]"] -row $row -sticky w
	}
    }
    set nx 12
    foreach {name} {X Z} {
	if {[info exists keep($name)]} {
	    foreach value $keep($name) {
		incr nx
		grid [label $w.x.x$nx -text "[tkj-get-field-label $name]: $value"] -row $nx -sticky w
	    }
	}
    }
    set nd 0
    foreach name {ON ONT1 ONT2 KUN KUNT1 KUNT2} {
	if {[info exists keep($name)]} {
	    grid [frame $w.d.f$nd -border 5 -relief ridge] -row $nd -sticky ew
	    set nv 0
	    #grid [label $w.d.f$nd.l$nv -text [tkj-get-field-label $name]] -row $nv -sticky ew
	    #incr nv
	    foreach value $keep($name) {
		grid [label $w.d.f$nd.l$nv -font [tkj-font 16] -text $value] -row $nv -sticky w
		incr nv
	    }
	    incr nd
	}
    }
    foreach name {ENG Y W} {
	if {[info exists keep($name)]} {
	    grid [frame $w.d.f$nd -border 5 -relief ridge] -row $nd -sticky ew
	    set nv 0
	    #grid [label $w.d.f$nd.l$nv -text [tkj-get-field-label $name]] -row $nv -sticky ew
	    #incr nv
	    foreach value $keep($name) {
		grid [label $w.d.f$nd.l$nv -text $value] -row $nv -sticky w
		incr nv
	    }
	    incr nd
	}
    }
}
#
# tkj-browse-select - implement the brower pick
#
proc tkj-browse-select {w} {
    upvar \#0 $w data
    tkj-browse-record [lindex $data(listbox) [$w.l curselection]] {Tk Kanji}
}
#
# tkj-browse-reload - fill the main selector with some index/select set
#
proc tkj-browse-reload {w} {
    global kanji
    upvar \#0 $w data
    
    tkj-status "reload select $data(select) index $data(index)"
    $w.l delete 0 end
    set data(listbox) {}
    foreach k [eval $data(select)] {
	set select($k) {}
    }
    foreach k [eval $data(index)] {
	if {[info exists select($k)]} {
	    lappend data(listbox) $k
	    $w.l insert end $k
	}
    }
    bind $w.l <Double-ButtonPress-1> [list tkj-browse-select $w]
    tkj-status ""
}
#
# browse a subset of the dictionaries loaded
#
proc kanji-browse {w} {
    upvar \#0 $w data
    wm title $w {Tk Kanji}
    tkj-empty $w
    # menu frame, listbox, scrollbar, and status
    grid [frame $w.m] -row 0 -column 0 -columnspan 2 -sticky ew
    grid [listbox $w.l -font [tkj-font 24] -yscrollcommand [list $w.y set]] -row 1 -column 0 -sticky nsew
    grid [scrollbar $w.y -command [list $w.l yview]] -row 1 -column 1 -sticky ns
    grid [label $w.s -textvar tkkanji(status)] -row 2 -columnspan 2 -sticky ew
    grid columnconfigure $w 0 -weight 1
    grid columnconfigure $w 1 -weight 0
    grid rowconfigure $w 0 -weight 0
    grid rowconfigure $w 1 -weight 1
    # file menu
    pack [tkj-file-menu $w.m.f] -side left
    # application menu
    pack [tkj-application-menu $w.m.a $w] -side left
    # index menu
    pack [tkj-index-menu $w.m.i ${w}(index) [list tkj-browse-reload $w]] -side left
    # select menu
    pack [tkj-select-menu $w.m.s ${w}(select) [list tkj-browse-reload $w]] -side left
    # listbox
    tkj-browse-reload $w
}
