########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, NM, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################

namespace eval ::config:: {
    array set default {
	kanjidic-load 1 kanjidic-path {kanjidic}
	kanjidic.doc-path {kanjidic.doc}
	kanjd212-load 0 kanjd212-path {kanjd212}
	kanjd212.doc-path {kanjd212.doc}
	font-48 {fixed}
	font-32 {fixed}
	font-24 {fixed}
	font-16 {fixed}
	subset All
	startup tkj-choose
    }
    set default(startup) [tkj-default application]
    array set explain {
	search {
	    When tkkanji starts it searches three places for a
	    configuration file:  the current directory, the directory
	    where the tkkanji script is stored, and the Tcl/Tk library 
	    directory.

	    If tkkanji finds a configuration file, and the file
	    specifies a valid configuration, then tkkanji will load
	    the specified dictionary subset and execute the startup
	    application.  If any of those steps fails, then tkkanji
	    should return you to this configuration page so you can
	    amend the problem, and write a valid configuration back to 
	    the disk.

	    This option menu lists the three places that tkkanji will
	    search for its configuration file.  Choose the one you
	    want to write before you press the Write button.
	}
	kanjidic-path {
	    This combination check button and file name entry
	    specifies whether tkkanji should load kanjidic and where
	    tkkanji can find kanjidic on your computer.

	    Click on "Kanjidic" to specify whether or not tkkanji
	    should look for and load kanjidic.  (The only reason you
	    might turn kanjidic loading off is if you want to view
	    kanjd212 and your computer doesn't have enough memory to
	    load both dictionaries at once.)

	    If you choose to load kanjidic, you can either type the
	    path name directly into the entry provided, or click on
	    the "Select" button to navigate through your folders to
	    find it.

	    Kanjidic is the primary Kanji dictionary file.  If you don't
	    already have a copy, you'll need to download one from:

		ftp://ftp.cc.monash.edu.au/pub/nihongo/00INDEX.html

	    or one of the mirror sites listed there.  The file is
	    named "kanjidic.gz", and you'll also need to decompress
	    the file, as explained on the download page.

	    Be warned that some combinations of web browsers and
	    servers will decompress the file automatically, or write a 
	    compressed file into a file name without the ".gz"
	    extension.
	}
	kanjidic.doc-path {
	    This specifies where the documentation for kanjidic is
	    stored on your computer.  This is optional and can be
	    downloaded from the same place you obtained kanjidic
	    itself.
	}
	kanjd212-path {
	    This specifies where kanjd212 is stored on your computer.
	    Kanjd212 is the Kanji dictionary defining ~6000 rarer
	    Kanji specified by JISX-0212.  This is optional and can be 
	    downloaded from the same place you obtained kanjidic
	    itself.
	}
	kanjd212.doc-path {
	    This specifies where the documentation for kanjd212 is
	    stored on your computer.  This is optional and can be
	    downloaded from the same place you obtained kanjidic
	    itself.
	}
	font {
	    Tkkanji chooses fonts according to the font family, the
	    font size, and the characters which need to be displayed.

	    Pressing on "Select" will allow you to preview the kanji
	    display for each available font family available on your
	    computer.

	    Many of the families listed do not define any Kanji
	    characters and will end up using the same Kanji fonts as
	    other families.

	    If none of the families look any good to you, then you
	    should head back to Jim Breen's web site,

		ftp://ftp.cc.monash.edu.au/pub/nihongo/00INDEX.html

	    and look for some fonts to download.
	}
	subset {
	    For many purposes, a subset of kanjidic is more than
	    enough.  These subsets are based on the Jouyou grade
	    levels, the standard Kanji sets taught at each grade
	    level in Japanese schools, the whole Jouyou list of Kanji
	    used in mass communication, and the Jinmeiyou list of
	    Kanji frequently used in names.
	}
	startup {
	    The startup option menu specifies the default application
	    which tkkanji launches after loading its dictionaries.
	}
    }
}
#
# return the list of config variables
#
proc ::config::vars {} {
    global ::config::default
    array names ::config::default
}
#
# initialize an empty configuration
#
proc ::config::init {} {
    global ::config::config
    global ::config::default
    array set ::config::config [array get ::config::default]
}
#
# check the contents of a configuration
# to see that each file to be loaded exists
# and is readable
#
proc ::config::check {} {
    global ::config::config
    foreach name [::config::vars] {
	if { ! [info exists ::config::config($name)]} {
	    set ::config::config(error) "no ::config::config($name)"
	    return 0
	}
    }
    foreach {check file} {
	kanjidic-load kanjidic-path
	kanjd212-load kanjd212-path
    } {
	if {$::config::config($check)} {
	    if {[file exists $::config::config($file)]
		&& [file readable $::config::config($file)]} continue
	    set ::config::config(error) "can't read $::config::config($file)"
	    return 0
	}
    }
    return 1
}
#
# list the files to check for configuration
#
proc ::config::search {} {
    list \
	[file join [pwd] tkkanji.cfg] \
	[file join [file dirname [info script]] tkkanji.cfg] \
	[file join [info library] tkkanji.cfg]
}
#
# look for a readable config file, load it,
# and return the results of checking its paths
#
proc ::config::load {} {
    global ::config::config
    foreach file [::config::search] {
	if {[file exists $file] && [file readable $file]} {
	    set ::config::config(file) $file
	    if {[catch {source $file}]} {
		break
	    } else {
		return [::config::check]
	    }
	}
    }
    ::config::init
    return 0
}
#
# update a configuration field
#
proc ::config::change {w cmd var} {
    global ::config::config
    switch $cmd {
	path -
	dict {
	    set f [tk_getOpenFile -parent $w]
	    if {[string length $f] > 0} {
		set ::config::config($var) $f
	    }
	}
	font {
	    set x [toplevel-window-name]
	    toplevel $x
	    wm title $x {Select Font}
	    upvar \#0 $x data
	    bind $x <Destroy> "catch {unset $x}"
	    # fetch font family
	    set data(family) $::config::config($var)
	    set data(families) [font families]
	    if {[lsearch [font families] $data(family)] < 0} {
		set data(family) [lindex $data(family) 0]
	    }
	    # fetch font size
	    set data(size) [lindex [split $var -] 1]
	    pack [frame $x.t] -side top
	    pack [label $x.t.l -text Family:] -side left
	    pack [label $x.t.f -textvar ${x}(family)] -side left
	    pack [frame $x.o] -side top
	    pack [listbox $x.o.l -height 5 -yscrollcommand [list $x.o.y set]] -side left -fill x -expand true
	    pack [scrollbar $x.o.y -orient vertical -command [list $x.o.l yview]] -side left -fill y
	    #pack [menubutton $x.o -textvar ${x}(family) -relief raised -indicatoron 1 -menu $x.o.m] -side top
	    #menu $x.o.m -tearoff no
	    foreach f $data(families) {
		#$x.o.m add radiobutton -label $f -var ${x}(family) -value $f -command [list $x.l configure -font [list $f $data(size)]]
		$x.o.l insert end $f
	    }
	    bind $x.o.l <Double-ButtonPress-1> [list ::config::change $w font-sel $x]
	    pack [label $x.l -text "\u4e00 \u4e8c\n\u4e09 \u56db\n\u4e94 \u516d\n\u4e03 \u516b\n\u4e5d \u5341"\
		      -font [list $data(family) $data(size)]] -side top
	    pack [frame $x.b] -side top
	    pack [button $x.b.okay -text Okay -command [list set ${x}(done) 1]] -side left
	    pack [button $x.b.cancel -text Cancel -command [list set ${x}(done) 0]] -side left
	    vwait ${x}(done)
	    if {$data(done)} {
		set ::config::config($var) $data(family)
	    }
	    destroy $x
	}
	font-sel {
	    set x $var
	    upvar \#0 $x data
	    set data(family) [lindex $data(families) [$x.o.l curselection]]
	    $x.l configure -font [list $data(family) $data(size)]
	}
	default {
	    error "invalid configuration style: $cmd"
	}
    }
}
#
# explain a configuration item
#
proc ::config::explain {var lbl} {
    global ::config::config
    global ::config::explain
    browsedata "Explain: $lbl" $::config::explain($var)
}
#
# write the configuration
#
proc ::config::write {} {
    global ::config::config
    if { ! [::config::check]} {
	error "There is an error in your configuration which needs to be corrected before you can continue:  $::config::config(error)"
    }
    set c "array set ::config::config {\n"
    foreach {name} [::config::vars] {
	append c "  {$name} {$::config::config($name)}\n"
    }
    append c "}\n"
    if {[catch {
	set fp [open $::config::config(file) w]
	puts -nonewline $fp $c
	close $fp
    } error]} {
	error "error writing configuration file to $::config::config(file): $error"
    }
    set ::config::config(done) 1
}
#
# start up from a configuration dialog
#
proc ::config::startup {} {
    global ::config::config
    if { ! [::config::check]} {
	error "There is an error in your configuration which needs to be corrected before you can continue:  $::config::config(error)"
    }
    set ::config::config(done) 1
}
#
# display the configuration
# and allow it to be modified,
# checked, executed, and saved
#
proc ::config::display {w state} {
    global ::config::config
    wm title $w {Tk Kanji Configuration}
    set f 0
    if {[string compare $state normal] == 0} {
	set m $w.f[incr f]
	pack [frame $m] -side top
	pack [tkj-file-menu $m.f] -side left
	pack [tkj-application-menu $m.a $w] -side left
    }
    foreach {lbl var cmd} {
	{Tk Kanji Configuration} {} none
	{Configuration file} file search
	{Kanjidic} kanjidic dict
	{Kanjidic.doc path} kanjidic.doc-path path
	{Kanjd212} kanjd212 dict
	{Kanjd212.doc path} kanjd212.doc-path path
	{48pt font family} font-48 font
	{32pt font family} font-32 font
	{24pt font family} font-24 font
	{16pt font family} font-16 font
	{Subset to read} subset subset
	{Startup application} startup startup
    } {
	pack [frame $w.f[incr f]] -side top -fill x
	switch $cmd {
	    none {
		pack [label $w.f$f.l -text $lbl] -side left
	    }
	    search {
		pack [label $w.f$f.l -text "$lbl: "] -side left -anchor w
		eval tk_optionMenu $w.f$f.o ::config::config($var) [::config::search]
		pack $w.f$f.o -side left -anchor w
		pack [button $w.f$f.help -text { ? } -command [list ::config::explain $var $lbl]] -side right -anchor e
	    }
	    dict {
		pack [checkbutton $w.f$f.b -text $lbl -var ::config::config($var-load)] -side left
		pack [button $w.f$f.help -text { ? } -command [list ::config::explain $var $lbl]] -side right -anchor e
		pack [button $w.f$f.c -text select -command [list ::config::change $w $cmd $var-path]] -side right -anchor e
		pack [entry $w.f$f.e -width 40 -textvar ::config::config($var-path)] -side right -anchor e
	    }
	    path -
	    font {
		pack [label $w.f$f.l -text "$lbl: "] -side left
		pack [button $w.f$f.help -text { ? } -command [list ::config::explain $var $lbl]] -side right -anchor e
		pack [button $w.f$f.c -text select -command [list ::config::change $w $cmd $var]] -side right -anchor e
		pack [entry $w.f$f.e -width 40 -textvar ::config::config($var)] -side right -anchor e
	    }
	    subset {
		pack [label $w.f$f.l -text $lbl:] -side left
		tk_optionMenu $w.f$f.o ::config::config($var) \
		    Jouyou-1 \
		    Jouyou-1-2 \
		    Jouyou-1-3 \
		    Jouyou-1-4 \
		    Jouyou-1-5 \
		    Jouyou-1-6 \
		    Jouyou \
		    Jouyou+Jinmeiyou \
		    JISX-0208 \
		    All
		pack $w.f$f.o -side left
		pack [button $w.f$f.help -text { ? } -command [list ::config::explain $var $lbl]] -side right -anchor e
	    }
	    startup {
		pack [label $w.f$f.l -text $lbl:] -side left
		eval tk_optionMenu $w.f$f.o ::config::config($var) [tkj-applications]
		pack $w.f$f.o -side left
		pack [button $w.f$f.help -text { ? } -command [list ::config::explain $var $lbl]] -side right -anchor e
	    }
	    default {
		error "unimplemented configuration style: $cmd"
	    }
	}
    }
    pack [button $w.b[incr f] -text Write -command ::config::write] -side top -fill x
    if {[string compare $state normal] != 0} {
	pack [button $w.b[incr f] -text Done -command ::config::startup] -side top -fill x
    }
    pack [label $w.status -textvar tkkanji(status)] -side top -fill x
    if {[string compare $state normal] != 0} {
	set ::config::config(done) 0
	vwait ::config::config(done)
	tkj-splash $w
    }
}
#
# wrap these pieces
#
proc tkj-configure {w {state normal}} {
    switch $state {
	startup {
	    if { ! [::config::load]} { ::config::display [tkj-empty $w] $state }
	}
	rescue -
	normal {
	    ::config::display [tkj-empty $w] $state
	}
    }
}
#
# access parts of the configuration
#
proc tkj-cget {key} { global ::config::config; set ::config::config($key) }
#
# return a font
#
proc tkj-font {pts} {
    # hack
    global tkj-override-font
    if {[info exists tkj-override-font($pts)]} {
	set tkj-override-font($pts)
    } else {
	list [tkj-cget font-$pts] $pts
    }
}
