########################################################################
#
# tk kanji configuration
#
# Copyright 1999 by Roger E. Critchlow Jr, Santa Fe, New Mexico, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################

########################################################################
#
# data for studying Heisig's Remembering the Kanji, I
# made especially convenient because the first english
# meaning in kanjidic appears to be the Heisig keyword.
#
########################################################################
namespace eval ::heisig:: {}

#
# the last character covered in each lesson
# using Heisig's ordering of the kanji
#
set ::heisig::lessons {
       0
      15   34   52   70   94
     104  126  172  194  234
     249  276  299  323  352
     369  395  475  508  514
     577  636  766  795  891
     950 1026 1044 1085 1124
    1183 1219 1247 1293 1332
    1394 1426 1483 1530 1586
    1615 1647 1681 1709 1756
    1775 1805 1827 1852 1879
    1903 1926 1977 2005 2025
    2042
}

#
# build a menu for selecting a subset of
# Heisig kanji for study
#
proc heisig-lesson-menu {m ms var cmd} {
    $m add cascade -label {Heisig Lessons} -menu $ms
    menu $ms -tearoff no
    # in groups of five
    for {set i 1} {$i < 55} {incr i 5} {
	set j [expr {$i+4}]
	if {$i == 51} {
	    incr j
	}
	set l "$i..$j ([heisig-count-range $i $j])"
	$ms add radiobutton -label $l -variable $var \
	    -value "heisig-expand-range $i $j" -command $cmd
    }
    # review from beginning
    for {set i 1} {$i < 55} {incr i 5} {
	set j [expr {$i+4}]
	if {$i == 51} {
	    incr j
	}
	if {$i == 1} {
	    $ms add command -label {} -state disabled -columnbreak 1
	} else {
	    set l "1..$j ([heisig-count-range 1 $j])"
	    $ms add radiobutton -label $l -variable $var \
		-value "heisig-expand-range 1 $j" -command $cmd
	}
    }
}

proc heisig-expand-range {i j} {
    global ::heisig::lessons
    set i [lindex $::heisig::lessons [incr i -1]]
    set j [lindex $::heisig::lessons $j]
    lrange [tkj-get-index L] $i $j
}

proc heisig-count-range {i j} {
    global ::heisig::lessons
    set i [lindex $::heisig::lessons [incr i -1]]
    set j [lindex $::heisig::lessons $j]
    expr {$j-$i}
}
