########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, NM, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################

#
# play concentration
#
proc concentration {w op} {
    upvar \#0 $w data
    set f concentration
    switch -glob $op {
	setup {
	    wm title $w {Tk Kanji-tration}
	    catch {unset data}
	    set data(delay) 2000
	    set data(size) 4
	    set data(select) [tkj-default select]

	    pack [frame $w.m] -side top -fill x
	    pack [menubutton $w.m.o -text {options} -menu $w.m.o.m] -side left
	    menu $w.m.o.m -tearoff no
	    $w.m.o.m add radiobutton -label {4x4 Game} -var ${w}(size) -value 4 -command [list $f $w resize]
	    $w.m.o.m add radiobutton -label {6x6 Game} -var ${w}(size) -value 6 -command [list $f $w resize]
	    $w.m.o.m add separator
	    foreach sec {1 2 5} {
		$w.m.o.m add radiobutton -label "Delay $sec sec" -variable ${w}(delay) -value ${sec}000
	    }
	    $w.m.o.m add separator
	    $w.m.o.m add command -label {Done} -command [list tkj-chooser $w]
	    pack [tkj-select-menu $w.m.s ${w}(select) [list $f $w start]] -side left
	    pack [button $w.m.n -text {new game} -command [list $f $w start]] -side left
	    pack [frame $w.g] -side top
	    pack [label $w.status -textvar tkkanji(status)] -side top -fill x
	    $f $w resize
	}
	resize {
	    tkj-empty $w.g
	    set data(rows) {}
	    set data(cols) {}
	    for {set i 0} {$i < $data(size)} {incr i} {
		lappend data(rows) $i
		lappend data(cols) $i
	    }
	    foreach i $data(rows) {
		foreach j $data(cols) {
		    grid [button $w.g.b$i$j -text {} -font [tkj-font 48] -width 2 \
			      -command [list $f $w [list show $i $j]]] \
			-in $w.g -row $i -column $j -sticky nsew
		}
	    }
	    set data(normal-bg) [$w.g.b$i$j cget -bg]
	    $f $w start
	}
	start {
	    set data(picked) {}
	    foreach i $data(rows) {
		foreach j $data(cols) {
		    $w.g.b$i$j configure -text {} -state normal -bg $data(normal-bg)
		}
	    }
	    set r [expr {-1+$data(size)*$data(size)/2}]
	    set data(kanji) [lrange [shuffle [eval $data(select)]] 0 $r]
	    set data(kanji) [shuffle [concat $data(kanji) $data(kanji)]]
	}
	show* {
	    foreach {show i j} $op break
	    switch  [llength $data(picked)] {
		0 {
		    lappend data(picked) $i $j
		    $w.g.b$i$j configure -text [lindex $data(kanji) [expr {$i*$data(size)+$j}]]
		    $w.g.b$i$j configure -state disabled
		}
		2 {
		    lappend data(picked) $i $j
		    $w.g.b$i$j configure -text [lindex $data(kanji) [expr {$i*$data(size)+$j}]]
		    $w.g.b$i$j configure -state disabled
		    foreach {pi pj} $data(picked) break
		    if {[string compare [$w.g.b$pi$pj cget -text] [$w.g.b$i$j cget -text]] == 0} {
			$w.g.b$pi$pj configure -bg white
			$w.g.b$i$j configure -bg white
			set data(picked) {}
		    } else {
			set data(timer) [after $data(delay) [list $f $w clear]]
		    }
		}
		default {
		    catch {after cancel $data(timer)}
		    $f $w clear
		    $f $w $op
		}
	    }
	}
	clear {
	    foreach {i j} $data(picked) {
		$w.g.b$i$j configure -text {}
		$w.g.b$i$j configure -state normal
	    }
	    set data(picked) {}
	}
    }
}

proc kanji-tration {w} {
    tkj-empty $w
    concentration $w setup
}