########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, New Mexico, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################
#
# every main window has a status line
# which mirrors this variable
#
proc tkj-status {msg} {
    global tkkanji
    set tmp $tkkanji(status)
    set tkkanji(status) $msg; update
    return $tmp
}
proc tkj-push-status {msg} {
    global tkkanji
    lappend tkkanji(status-stack) [tkj-status $msg]
}    
proc tkj-pop-status {} {
    global tkkanji
    tkj-status [lindex tkkanji(status-stack) end]
    set tkkanji(status-stack) [lreplace $tkkanji(status-stack) end end]
}
#
# simply quit
#
proc tkj-quit {} {
    destroy .
}
#
# display an error and its Tcl stack trace
#
proc tkj-view-trace {title error} {
    global errorInfo
     browsedata $title "Error message: {\n$error\n}\nStack trace: {\n$errorInfo\n}"
}
#
# do the right thing
#
proc tkj-view-copyright {} {
    browsedata {Tk Kanji Copyright} [tkj-copyright]
}
proc tkj-view-terms {} {
    browsedata {Tk Kanji Terms} [gpl-terms]
}
proc tkj-view-warranty {} {
    browsedata {Tk Kanji Warranty} [gpl-warranty]
}
proc tkj-view-license {} {
    browsedata {Tk Kanji License} [gpl-license]
}
#
# empty a window of its contents
# NB, toplevel windows are children of .
#
proc tkj-empty {w} {
    foreach child [winfo children $w] {
	destroy $child
    }
    return $w
}
#
# make a new toplevel window with title
#
proc tkj-toplevel {title} {
    set w [toplevel-window-name]
    toplevel $w
    wm title $w $title
    return $w
}
#
# make the file menu
#
proc tkj-file-menu {w} {
    menubutton $w -text file -menu $w.m
    menu $w.m -tearoff no
    $w.m add command -label {view kanjidic} \
	-command [list browsefile kanjidic [tkj-cget kanjidic-path] [tkj-font 16]]
    $w.m add command -label {view kanjidic.doc} \
	-command [list browsefile kanjidic.doc [tkj-cget kanjidic.doc-path] [tkj-font 16]]
    $w.m add command -label {view kanjd212} \
	-command [list browsefile kanjd212 [tkj-cget kanjd212-path] [tkj-font 16]]
    $w.m add command -label {view kanjd212.doc} \
	-command [list browsefile kanjd212.doc [tkj-cget kanjd212.doc-path] [tkj-font 16]]
    $w.m add separator
    $w.m add command -label {view copyright} -command {browsedata {Tk Kanji Copyright} [tkj-copyright]}
    $w.m add command -label {view terms} -command {browsedata {Tk Kanji Terms} [gpl-terms]}
    $w.m add command -label {view warranty} -command {browsedata {Tk Kanji Warranty} [gpl-warranty]}
    $w.m add command -label {view license} -command {browsedata {Tk Kanji License} [gpl-license]}
    $w.m add separator
    $w.m add command -label quit -command {tkj-quit}
    return $w
}
#
# make the application menu
#
proc tkj-application-menu {w win} {
    menubutton $w -text applications -menu $w.m
    menu $w.m -tearoff no
    foreach app [tkj-applications] {
	$w.m add command -label $app -command [list $app $win]
    }
    return $w
}
#
#
#
proc tkj-index-menu {w varname cmd} {
    upvar \#0 $varname index
    set index [tkj-default index]
    menubutton $w -text index -menu $w.m
    menu $w.m -tearoff no
    $w.m add radiobutton -label All -variable $varname -value "tkj-get-index all" -command $cmd
    foreach {code} {F DR H N V E IN K L MN MP O J0 J1 U} {
	$w.m add radiobutton -label [tkj-get-field-label $code] -variable $varname -value "tkj-get-index $code" -command $cmd
    }
    return $w
}
#
#
#
proc tkj-select-menu {w varname cmd} {
    upvar \#0 $varname select
    set select [tkj-default select]
    menubutton $w -text select -menu $w.m
    menu $w.m -tearoff no
    $w.m add radiobutton -label All -variable $varname -value "tkj-get-class all" -command $cmd
    # defer on  P I Q
    foreach {code} {G B C S L} {
	tkj-select-submenu $w.m $varname $cmd $code
    }
    return $w
}
#
# tkj-select-menu - build a select menu
#   choose, via hierarchical menues, one class to highlight
#
proc tkj-select-submenu {m varname cmd code} {
    set ms $m.[string tolower $code]
    switch $code {
	G {
	    # JouYou grades
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    foreach {code} {G G1 G2 G3 G4 G5 G6 G8 G9} {
		foreach {k v} [split $code {}] break
		$ms add radiobutton -label "[tkj-get-field-value-label $k $v] ([tkj-class-size $code])" \
		    -variable $varname -value "tkj-get-class $code" \
		    -command $cmd
	    }
	}
	B - C {
	    # Radicals
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    foreach {i j} {1 49 50 99 100 149 150 199 200 214} {
		$ms add cascade -label "$i-$j" -menu $ms.s$i
		menu $ms.s$i -tearoff no
		if {$i == 1} { set dk 9 } else { set dk 10 }
		for {set k $i} {$k <= $j} {incr k $dk; set dk 10} {
		    $ms.s$i add cascade -label "$k-[expr {$k+$dk-1}]" -menu $ms.s$i.s$k
		    menu $ms.s$i.s$k -tearoff no
		    for {set l $k} {$l < $k+$dk} {incr l} {
			if {[tkj-class-exists $code$l]} {
			    $ms.s$i.s$k add radiobutton -label "$l ([tkj-class-size $code$l])" \
				-variable $varname -value "tkj-get-class $code$l" \
				-command $cmd
			}
		    }
		}
	    }
	}
	S {
	    # Stroke counts
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    foreach i [tkj-class-names S?*] {
		set stroke([string range $i 1 end]) 1
	    }
	    set cb 0
	    foreach i [lsort -integer [array names stroke]] {
		$ms add radiobutton -label "$i ([tkj-class-size S$i])" \
		    -variable $varname -value "tkj-get-class S$i" \
		    -command $cmd -columnbreak $cb
		set cb [expr {$i % 10 == 0}]
	    }
	}
	L {
	    # Heisig lessons
	    heisig-lesson-menu $m $ms $varname $cmd
	}
	P {
	    # Skip codes
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    foreach i [tkj-class-names P?*] {
		foreach {s1 s2 s3} [split [string range $i 1 end] -] break
		set s1 [format 0%02o $s1]
		set s2 [format 0%02o $s2]
		set s3 [format 0%02o $s3]
		set skipdata($s1) 1
		set skipdata($s1$s2) 1
		set skipdata($s1$s2$s3) 1
	    }
	    foreach s1 [lsort [array names skipdata ???]] {
		$ms add cascade -label [format %d-*-* $s1] -menu $ms.s$s1
		menu $ms.s$s1 -tearoff no
		foreach s1s2 [lsort [array names skipdata $s1???]] {
		    set s2 [string range $s1s2 3 end]
		    $ms.s$s1 add cascade -label [format %d-%d-* $s1 $s2] -menu $ms.s$s1.s$s2
		    menu $ms.s$s1.s$s2 -tearoff no
		    foreach s1s2s3 [lsort [array names skipdata $s1s2???]] {
			set s3 [string range $s1s2s3 6 end]
			set p [format %d-%d-%d $s1 $s2 $s3]
			$ms.s$s1.s$s2 add radiobutton -label "$p ([tkj-class-size P$p])" \
			    -variable $varname -value "tkj-get-class P$p" \
			    -command $cmd
		    }
		}
	    }
	}
	I {
	    # Spahn & Hadamitzky codes
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    scan a %c a
	    foreach i [lsort [tkj-class-names {I[1-9]*}]] {
		if { ! [regexp {^.([0-9]+)([a-z])([0-9]+)$} $i all s1 s2 s3]} {
		    puts "S&H $i didn't scan"
		}
		set s1 [format %03o $s1]
		scan $s2 %c s2
		set s2 [format %03o [expr {$s2-$a}]]
		set s3 [format %03o $s3]
		set sethdata($s1) 1
		set sethdata($s1$s2) 1
		set sethdata($s1$s2$s3) 1
	    }
	    # 
	    foreach s1 [lsort [array names sethdata ???]] {
		$ms add cascade -label [format %d** $s1] -menu $ms.s$s1
		menu $ms.s$s1 -tearoff no
		foreach s1s2 [lsort [array names sethdata $s1???]] {
		    set s2 [string range $s1s2 3 end]
		    set a2 [expr {$a+$s2}]
		    $ms.s$s1 add cascade -label [format %d%c* $s1 $a2] -menu $ms.s$s1.s$s2
		    menu $ms.s$s1.s$s2 -tearoff no
		    foreach s1s2s3 [lsort [array names sethdata $s1s2???]] {
			set s3 [string range $s1s2s3 6 end]
			set p [format %d%c%d $s1 $a2 $s3]
			$ms.s$s1.s$s2 add radiobutton -label "$p ([tkj-class-size I$p])" \
			    -variable $varname -value "tkj-get-class I$p" \
			    -command $cmd
		    }
		}
	    }
	}
	Q {
	    # Four Corner codes
	    $m add cascade -label [tkj-get-field-label $code] -menu $ms
	    menu $ms -tearoff no
	    foreach i [lsort [tkj-class-names {Q????}]] {
		foreach {q q1 q2 q3 q4} [split $i {}] break
		if {[catch {$ms entrycget $q1??? -label} msg]} {
		    $ms add cascade -label $q1??? -menu $ms.q$q1
		    menu $ms.q$q1 -tearoff no
		}
		if {[catch {$ms.q$q1 entrycget $q1$q2?? -label} msg]} {
		    $ms.q$q1 add cascade -label $q1$q2?? -menu $ms.q$q1.q$q2
		    menu $ms.q$q1.q$q2 -tearoff no
		}
		if {[catch {$ms.q$q1.q$q2 entrycget $q1$q2$q3? -label} msg]} {
		    $ms.q$q1.q$q2 add cascade -label $q1$q2$q3? -menu $ms.q$q1.q$q2.q$q3
		    menu $ms.q$q1.q$q2.q$q3 -tearoff no
		}
		set p $q1$q2$q3$q4
		$ms.q$q1.q$q2.q$q3 add radiobutton -label "$p ([tkj-class-size Q$p])" \
		    -variable $varname -value "tkj-get-class Q$p" \
		    -command $cmd
	    }
	}
	default {
	    error "undefined submenu in tkj-select-submenu: $code"
	}
    }
}
#
# chooser application
#
proc tkj-chooser {w} {
    tkj-empty $w
    wm title $w {Tk Kanji Chooser}
    set n 0
    foreach a [tkj-applications] {
	pack [button $w.b[incr n] -text $a -command [list $a $w]] -side top -fill x
    }
    pack [button $w.b[incr n] -text tkj-quit -command tkj-quit] -side top -fill x
    pack [label $w.l -textvar tkkanji(status)]
}