#!/usr/local/bin/wish

########################################################################
#
# Copyright (c) 1999 by Roger E. Critchlow Jr, Santa Fe, NM, USA,
# rec@elf.org -- http://www.elf.org
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
########################################################################

#
# tkkanji.tcl depends on the existence of three additional packages
#
#	tcl/tk - the Tool Command Language and UI Tool Kit,
#		freely available from http://www.scriptics.com
#		for windows, macintosh, and unix.
#		tkkanji was written for version 8.1.1, but any
#		more recent should be okay.
#
#	japanese fonts - appropriate to your system,
#		freely available from
#		ftp://ftp.cc.monash.edu.au/pub/nihongo/00INDEX.html
#
#	kanjidic - a dictionary of information about Kanji,
#		freely available from
#		ftp://ftp.cc.monash.edu.au/pub/nihongo/00INDEX.html
#
# Although everything is freely available, there may be restrictions
# on use and/or redistribution attached to these packages or to parts
# of them.  Read the label.
#
########################################################################

########################################################################
#
# find our library first
#
########################################################################

if { ! [info exists env(TKJ_LIB)]} {
    set env(TKJ_LIB) [file join [file dirname [info script]] tkj]
}
set auto_path [linsert $auto_path 0 $env(TKJ_LIB)]

########################################################################
#
# main program
#
########################################################################

proc tkj-splash {w} {
    tkj-empty $w
    wm title $w {Tk Kanji}
    grid [label $w.l0 -text "Tk \u6f22\u5b57" -font {fixed 48} -border 10] -row 0
    grid [label $w.l1 -text {Tk Kanji Version 0.1}] -row 1
    grid [label $w.l2 -text {Copyright (c) 1999, Roger E Critchlow Jr}] -row 2
    grid [label $w.l3 -text {Santa Fe, New Mexico, USA}] -row 3
    grid [label $w.l4 -text {rec@elf.org -- http://elf.org}] -row 4
    grid [button $w.l5 -text {Tk Kanji comes with ABSOLUTELY NO WARRANTY} -command tkj-view-warranty] -row 5
    grid [button $w.l6 \
	      -text "This is free software, and you are welcome\nto redistribute it under certain conditions" \
	      -command tkj-view-terms] -row 6
    grid [label $w.l7 -textvar tkkanji(status)] -row 7
}

wm withdraw .
toplevel .w
bind .w <Destroy> {
    if {[string compare %W .w] == 0} {
	destroy .
    }
}

srandom [clock clicks]
tkj-splash .w
tkj-configure .w startup

set nfailed 0
while {[catch {
    if {[tkj-cget kanjidic-load]} {
	tkj-load-dictionary [tkj-cget kanjidic-path] J0
    }
    if {[tkj-cget kanjd212-load]} {
	tkj-load-dictionary [tkj-cget kanjd212-path] J1
    }
    [tkj-cget startup] .w
} error]} {
    incr nfailed
    while {1} {
	switch [tk_dialog .dialog {Tk Kanji Startup Error} \
		    "Tk Kanji startup failed with the error message:\n$error" \
		    {} 0 {Reconfigure} {Quit} {Show Trace}] {
			0 {
			    tkj-configure .w rescue
			    break
			}
			1 {
			    tkj-quit
			}
			2 {
			    tkj-view-trace {Tk Kanji Startup Error Trace} $error
			    continue
			}
		    }
    }
}
    
